@@ -35,7 +35,7 @@ def default_config():
         config['DEFAULT'] = {'Name': '',
                              'Pen_Size': '5',
                              'Eraser_Size': '5',
-                             'Name_Heading': 'True',
+                             'Name_Heading': 'False',
                              'Default_Font': 'Sans Serif',
                              'Default_Style': 'Normal',
                              'Default_Font_Size': '12'}
@@ -181,6 +181,8 @@ class Canvas(QLabel):
             self.inactiveLayers[i] = QtGui.QPixmap(size)
             self.inactiveLayers[i].fill(Qt.transparent)
             self.layer_painter.begin(self.inactiveLayers[i])
+        self.layer_painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
+        self.layer_painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
         self.layer_painter.drawPixmap(temp.rect(), temp, temp.rect())
         self.layer_painter.end()
 
@@ -217,8 +219,7 @@ class Canvas(QLabel):
             self.painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
             self.painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
             self.painter.setPen(self.current_utensil.pen())
-            #self.painter.drawPoint(event.pos())
-
+            self.painter.drawPoint(event.pos())
 
             self.activeLayers.append(new_canvas_layer)
             self.inactiveLayers.clear()
@@ -227,7 +228,6 @@ class Canvas(QLabel):
             self.layer_change.emit()
 
             self.last_point_draw = event.pos()
-            ##self.update()
             self.painter.end()
         elif event.button() == Qt.MiddleButton:
             self.setCursor(Qt.ClosedHandCursor)
@@ -257,7 +257,6 @@ class Canvas(QLabel):
             if not self.current_utensil == Utensils.HIGHLIGHTER:
                 self.painter.drawLine(self.last_point_draw, event.pos())
                 self.last_point_draw = event.pos()
-            ##self.update()
             self.paintMirrorEvent()
             self.painter.end()
         elif event.buttons() and Qt.MiddleButton and self.mouse_button_scrolling:
@@ -286,95 +285,6 @@ class Canvas(QLabel):
             self.cursor_update()
             self.mouse_button_scrolling = False
 
-
-    # def mousePressEvent(self, event: QtGui.QMouseEvent) -> None:
-    #     if event.button() == Qt.LeftButton:
-    #         self.utensil_press = True
-    #         # self.numLayers += 1
-    #         new_canvas_layer = QtGui.QPixmap(self.size())
-    #         new_canvas_layer.fill(Qt.transparent)
-    #
-    #         #self.painter.begin(new_canvas_layer)
-    #         #self.painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-    #         #self.painter.setPen(self.current_utensil.pen())
-    #
-    #         #self.drawing_path_part = QtGui.QPainterPath()
-    #         # drawing_path = QtGui.QPainterPath()
-    #         # drawing_path.moveTo(event.pos())
-    #         self.drawing_path_part = QtGui.QPainterPath()
-    #         self.drawing_path_part.moveTo(event.pos())
-    #         # self.drawing_path_layers.append(drawing_path)
-    #
-    #         #self.painter.drawPoint(event.pos())
-    #
-    #         self.activeLayers.append(new_canvas_layer)
-    #         self.last_point_draw = event.pos()
-    #         self.update()
-    #         # self.painter.end()
-    #     elif event.button() == Qt.MiddleButton:
-    #         self.setCursor(Qt.ClosedHandCursor)
-    #         self.last_point_scroll = event.globalPos()
-    #         self.mouse_button_scrolling = True
-    #     else:
-    #         super(Canvas, self).mousePressEvent(event)
-    #
-    # def mouseMoveEvent(self, event: QtGui.QMouseEvent) -> None:
-    #     if event.buttons() and Qt.LeftButton and self.utensil_press:
-    #         x = self.width() - event.pos().x()
-    #         y = self.height() - event.pos().y()
-    #         if x < 200:
-    #             if y < 200:
-    #                 self.resizeLayer(-1, QSize(self.width() + 100, self.height() + 100))
-    #                 self.resizeCanvas(QSize(self.width() + 100, self.height()))
-    #             else:
-    #                 self.resizeLayer(-1, QSize(self.width() + 100, self.height()))
-    #                 self.resizeCanvas(QSize(self.width() + 100, self.height()))
-    #         elif y < 200:
-    #             self.resizeLayer(-1, QSize(self.width(), self.height() + 100))
-    #             self.resizeCanvas(QSize(self.width(), self.height() + 100))
-    #         #self.painter.begin(self.activeLayers[len(self.activeLayers) - 1])
-    #         #self.painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-    #
-    #         #self.painter.setPen(self.current_utensil.pen())
-    #         # self.painter.drawLine(self.last_point_draw, event.pos())
-    #         #self.drawing_path_part.closeSubpath()
-    #         self.drawing_path_part.lineTo(event.pos())
-    #         self.painter.begin(self.activeLayers[len(self.activeLayers) - 1])
-    #         #self.painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-    #         self.painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-    #         self.painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
-    #         self.painter.setPen(self.current_utensil.pen())
-    #         #self.drawing_path_layers[len(self.drawing_path_layers)-1].addPath(self.drawing_path_part)
-    #         self.painter.drawPath(self.drawing_path_part) #self.drawing_path_layers[len(self.drawing_path_layers)-1]
-    #         #self.drawing_path_part.moveTo(event.pos())
-    #         # self.drawingPath = None
-    #         self.update()
-    #         self.painter.end()
-    #
-    #         # self.last_point_draw = event.pos()
-    #         # self.update()
-    #         #self.painter.end()
-    #     elif event.buttons() and Qt.MiddleButton and self.mouse_button_scrolling:
-    #         offset = self.last_point_scroll - event.globalPos()
-    #         self.last_point_scroll = event.globalPos()
-    #         self.mouse_grab.emit(offset)
-    #     else:
-    #         super(Canvas, self).mouseMoveEvent(event)
-    #
-    # def mouseReleaseEvent(self, event: QtGui.QMouseEvent) -> None:
-    #     if event.button() == Qt.LeftButton:
-    #         self.utensil_press = False
-    #         # self.painter.begin(self.activeLayers[len(self.activeLayers) - 1])
-    #         # self.painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-    #         # self.painter.setPen(self.current_utensil.pen())
-    #         # self.painter.drawPath(self.drawingPath)
-    #         # #self.drawingPath = None
-    #         # self.update()
-    #         # self.painter.end()
-    #     elif event.button() == Qt.MiddleButton:
-    #         self.setCursor(self.cursor)
-    #         self.mouse_button_scrolling = False
-
     def wheelEvent(self, event: QtGui.QWheelEvent) -> None:
         self.scrolled.emit(event)
 
@@ -382,44 +292,17 @@ class Canvas(QLabel):
         self.layer_painter.begin(self.activeLayers[0])
         self.layer_painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
         self.layer_painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
-        # print(len(self.activeLayers))
-        self.layer_painter.drawPixmap(self.activeLayers[0].rect(), self.activeLayers[len(self.activeLayers) - 1])
-        # for i in range(1, len(self.activeLayers)):
-        #    self.layer_painter.drawPixmap(self.activeLayers[0].rect(), self.activeLayers[i])
+        self.layer_painter.drawPixmap(self.activeLayers[len(self.activeLayers) - 1].rect(), self.activeLayers[len(self.activeLayers) - 1], self.activeLayers[len(self.activeLayers) - 1].rect())
         self.layer_painter.end()
         self.setPixmap(self.activeLayers[0])
 
-    # def paintEvent(self, event):
-    #     #self.activeLayers[0].fill(Qt.white)
-    #     self.layer_painter.begin(self.activeLayers[0])
-    #     self.layer_painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-    #     self.layer_painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
-    #     #print(len(self.activeLayers))
-    #     self.layer_painter.drawPixmap(self.activeLayers[0].rect(), self.activeLayers[len(self.activeLayers)-1])
-    #     #for i in range(1, len(self.activeLayers)):
-    #     #    self.layer_painter.drawPixmap(self.activeLayers[0].rect(), self.activeLayers[i])
-    #     self.layer_painter.end()
-    #     self.setPixmap(self.activeLayers[0])
-    #     super(Canvas, self).paintEvent(event)
-
-    # def paintEvent(self, event):
-    #     self.activeLayers[0].fill(Qt.white)
-    #     self.layer_painter.begin(self.activeLayers[0])
-    #     self.layer_painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-    #     self.layer_painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
-    #     for i in range(1, len(self.activeLayers)):
-    #         self.layer_painter.drawPixmap(self.activeLayers[0].rect(), self.activeLayers[i])
-    #     self.layer_painter.end()
-    #     self.setPixmap(self.activeLayers[0])
-    #     super(Canvas, self).paintEvent(event)
-
     def hasChanged(self):
         if self.last_save is None:
             return False
         if len(self.floatingWidgets) > 0:
             return True
-        # elif len(self.floatingWidgets) == 1 and not self.floatingWidgets[0].is_heading:
-        #     return True
+        if len(self.activeLayers) == 1:
+            return False
         return not self.pixmap().toImage() == self.last_save
 
     def floatingWidgetPlace(self, widget):
@@ -460,12 +343,12 @@ class Canvas(QLabel):
     # Button click handling
     def clear(self):
         # Reset canvas
-        clear_canvas_layer = QtGui.QPixmap(self.size())
+        clear_canvas_layer = QtGui.QPixmap(self.activeLayers[0].size())
         clear_canvas_layer.fill(Qt.white)
         self.activeLayers.append(clear_canvas_layer)
         self.inactiveLayers.clear()
         self.paintMirrorEvent()
-        self.resizeCanvas(self.startSize)
+        # self.resizeCanvas(self.startSize)
         self.layer_change.emit()
         # Reset back to pen tool (done outside)
         # self.setUtensil(Utensils.PEN)
@@ -476,7 +359,7 @@ class Canvas(QLabel):
         self.layer_painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
         self.layer_painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
         for i in range(1, len(self.activeLayers)):
-            self.layer_painter.drawPixmap(self.activeLayers[0].rect(), self.activeLayers[i])
+            self.layer_painter.drawPixmap(self.activeLayers[i].rect(), self.activeLayers[i], self.activeLayers[i].rect())
         self.layer_painter.end()
         self.setPixmap(self.activeLayers[0])
 
@@ -485,7 +368,7 @@ class Canvas(QLabel):
         if len(self.activeLayers) > 1:
             self.inactiveLayers.append(self.activeLayers.pop())
             self.change_master_layer()
-            self.paintMirrorEvent()
+            # self.paintMirrorEvent()
             self.layer_change.emit()
 
     def redo(self): # Now finishing command pattern
@@ -527,6 +410,8 @@ class CanvasWindow(QScrollArea):
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
         self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
 
+        self.was_opened = False
+
     def mouseGrabScroll(self, offset):
         x = self.horizontalScrollBar().value() + offset.x()
         y = self.verticalScrollBar().value() + offset.y()
@@ -687,9 +572,10 @@ class UINoteWindow(QWidget):
         self.file_path_2 = ""
 
         # Add Heading if its on by default
-        dialog = QtWidgets.QDialog(self)
-        if default_heading == "True":
-            self.accept_header("---", True, dialog)
+        if not self.canvas_window.was_opened:
+            dialog = QtWidgets.QDialog(self)
+            if default_heading == "True":
+                self.accept_header("---", True, dialog)
 
     ########### Closing ###########
     def closeEvent(self, event):
@@ -702,7 +588,6 @@ class UINoteWindow(QWidget):
             else:
                 event.ignore()
 
-
     def keyPressEvent(self, event: QtGui.QKeyEvent) -> None:
         if event.key() == Qt.Key_BracketLeft:
             self.canvas_window.label.current_utensil.decrementWidth()
@@ -714,6 +599,10 @@ class UINoteWindow(QWidget):
             #for u in Utensils:
             #    u.incrementWidth()
             self.canvas_window.label.cursor_update()
+        elif event.key() == Qt.Key_Z and event.modifiers() == Qt.ControlModifier:
+            self.canvas_window.label.undo()
+        elif event.key() == Qt.Key_Y and event.modifiers() == Qt.ControlModifier:
+            self.canvas_window.label.redo()
 
     def saveOCRPopup(self):
         self.save_prompt = QtWidgets.QDialog(self)
@@ -1147,6 +1036,7 @@ class UINoteWindow(QWidget):
         self.open_directory(self.file_path)
         self.setWindowTitle(self.file_path)
 
+
     def update_open_recent_menu(self):
         try:
             with open(DIRECTORY_FILE, "r") as f:
@@ -1192,8 +1082,21 @@ class UINoteWindow(QWidget):
                         f.writelines(p + "\n" for p in paths)
                 self.deleted_file.emit(file_path)
                 self.deleteLater()
+
         else:
             self.canvas_window.label.loadImage(file_path)
+            self.canvas_window.was_opened = True
+            for i in range(len(self.canvas_window.label.floatingWidgets)):
+                if self.canvas_window.label.floatingWidgets[i].is_heading:
+                    self.canvas_window.label.floatingWidgets[i].to_delete = True
+                    self.canvas_window.label.floatingWidgetDelete(self.canvas_window.label.floatingWidgets[i])
+                    break
+            # while len(self.canvas_window.label.floatingWidgets) != 0:
+            #     self.canvas_window.label.floatingWidgets.pop().deleteLater()
+                # widget.deleteLater()
+            # for i in self.canvas_window.label.floatingWidgets:
+            #     i.deleteLater()
+            # self.canvas_window.label.floatingWidgets.clear()
 
     def ocr(self):
         if self.canvas_window.label.hasChanged() or self.file_path == "":
